/*
 * =====================================================================================
 *
 *       Filename:  rfid.cpp
 *
 *    Description:  RFID READ/WRITE class
 *    				ISO15693 PROTOCOL
 *
 *        Version:  1.0
 *        Created:  2008년 09월 01일 20시 45분 30초
 *       Revision:  none
 *       Compiler:  g++
 *
 *         Author:  Sangwoo Kim, nanum99@korea.com 
 *        Company:  my-company
 *
 * =====================================================================================
 */
#include "rfid.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

CRfid::CRfid(int dev)
{
	m_device = dev;
	m_portNo = -1;
	sprintf(m_deviceName, "/dev/ttyS%d",dev);
	memset(m_sn, 0x00, MAX_SN_SIZE); 
	memset(m_snString, 0x00, MAX_SN_STRING_SIZE); 
	memset(m_data, 0x00, MAX_DATA_SIZE);
}

CRfid::~CRfid()
{

}

void CRfid::rfOpen()
{
	struct termios newTerm;
	struct termios oldTerm;

	m_portNo = open(m_deviceName, O_RDWR | O_NOCTTY | O_NONBLOCK);

	if( m_portNo < 1) 
	{
		fprintf(stderr, "%s open error\n", m_deviceName);
		return;
	}
	
	tcgetattr(m_portNo, &oldTerm);
	memset(&newTerm, 0x00, sizeof(newTerm));

	newTerm.c_cflag = B38400 | CS8 | CLOCAL | CREAD;
	newTerm.c_cc[VTIME] = 0;
	newTerm.c_cc[VMIN] = 1;
	newTerm.c_iflag = IGNPAR | ICRNL;
	newTerm.c_oflag = 0;
	tcflush(m_portNo, TCIFLUSH);

	tcsetattr(m_portNo, TCSANOW, &newTerm);

}
void CRfid::readSerial()
{
	unsigned short isoBuffer[] = { 0x07, 0xFF, 0xB0, 0x01, 0x00, 0x1C, 0x56};
	unsigned char readBuffer[128] = {0x00,};

	int res=0, len=0;
	res = write(m_portNo, isoBuffer, sizeof(isoBuffer));
	if( res < 1)
	{
		fprintf(stderr, "readSerial : iso error\n");
		return;
	}

	usleep(500000);		// 0.5 secs

	res = read(m_portNo, readBuffer, sizeof(readBuffer));
	if( res < 1)
	{
		fprintf(stderr, "readSerial : read buffer error\n");
		return;
	}

	memcpy(m_sn, readBuffer+7, MAX_SN_SIZE);
	hexSnToStr(m_sn, m_snString);
}

unsigned char * CRfid::getSerialNo()
{
	return m_sn;
}

char * CRfid::getSNString()
{
	return m_snString;
}

/*
 * Serial(HEX value) -> STRING
 * @param
 *  srcStr - source string
 *  dstStr - destiniation string
 * @return
 *  변환 여부
 */
int CRfid::hexSnToStr(unsigned char * srcStr, char * dstStr)
{
	assert(dstStr!=NULL);
	assert(srcStr!=NULL);

	sprintf(dstStr, "%02X%02X%02X%02X%02X%02X%02X%02X",
					srcStr[0],srcStr[1],srcStr[2],srcStr[3],
					srcStr[4],srcStr[5],srcStr[6],srcStr[7]);

	return SUCCESS;
}

unsigned short CRfid::calcCRC16(unsigned short * pdata, int size)
{
	int i, j;
	unsigned short crcVal = 0xFFFF;
	for(i=0; i<size; ++i)
	{
		crcVal ^=(pdata[i] & 0xFF);

		for(j=0; j< 8; ++j)
		{
			if(crcVal & 0x0001)
				crcVal = ((crcVal >> 1) ^ 0x8408) & 0xFFFF;
			else
				crcVal = (crcVal >> 1);
		}
	}
	return crcVal;
}

bool CRfid::checkCRC16(unsigned short * pdata, int size)
{
	unsigned int crcVal;
	if( *pdata != 0x00)
	{
		crcVal = calcCRC16(pdata, size - 2);
		if((crcVal & 0x00FF) == (unsigned int)(pdata[size-2] & 0x00FF))
			if(((crcVal & 0xFF00) >> 8) == (unsigned int)(pdata[size-1] & 0x00FF))
				return true;
	}
	return false;
}

/*
 * 1. RF 태그로부터 시리얼 번호 읽기
 * 2. 시리얼 번호 명령어셋에 설정
 * 3. CRC16 설정
 * 4. RF 리더기로 명령어 전송
 */
void CRfid::setIso(unsigned short * isoBuffer, int size)
{
	int i;
	unsigned short crcVal = 0x00;

	assert(isoBuffer != NULL);

	readSerial();			
	
	for(i=0; i<MAX_SN_SIZE; ++i)
	{
		isoBuffer[i+5] = m_sn[i];
	}
	crcVal = calcCRC16(isoBuffer, size-2);
	isoBuffer[size-2] = (crcVal & 0x00FF);
	isoBuffer[size-1] = ((crcVal & 0xFF00) >> 8); 
}

void CRfid::readDataBlock(unsigned short start, unsigned short end)
{
	unsigned short isoBuffer[]= { 0x11, 0xff, 0xb0, 0x23, 0x01,	//기본셋팅 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	//어떤카드든 받기위해 시리얼 넘버를 0x00으로 셋팅
		0x00, 0x04, 		//0x00부터 4블럭을 읽어라
		0x00, 0x00 };	// CRC

	unsigned char readBuffer[256] = {0x00,};	

	isoBuffer[13] = start;
	isoBuffer[14] = end;

	setIso(isoBuffer,17);

	int res=0, len=0;

	res = write(m_portNo, isoBuffer, sizeof(isoBuffer));
	if( res < 1)
	{
		fprintf(stderr, "readDataBlock : write error\n");
		return;
	}

	usleep(500000);		// 0.05 secs 
	res = read(m_portNo, readBuffer, sizeof(readBuffer));
	if( res < 1)
	{
		fprintf(stderr, "readDataBlock : read error\n");
		return;
	}
	memset(m_data, 0x00, MAX_DATA_SIZE);
	memcpy(m_data, readBuffer+7, BLOCK_SIZE);
	memcpy(m_data + BLOCK_SIZE, readBuffer+7 + BLOCK_SIZE + 1, BLOCK_SIZE);
}

char * CRfid::getData()
{
	return m_data;
}

bool CRfid::isSerial()
{
	// 아무런 값도 없으면 false
	bool res = (bool) strlen( m_snString );
	
	// 값이 있지만, "0000000000000000"일때도 false
	if ( res )
	{
		res = (bool) strcmp( m_snString, "0000000000000000" );
	}
	
	return res;
}
